/*
	GraphFltCmd.h - draw graph from floating point data

	Will Eighteen, 2000: will@opaline.demon.co.uk

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	See COPYING for full details.
*/


#ifndef GRAPHFLTCMD_H
#define GRAPHFLTCMD_H

#include "NoUndoCmd.h"
#include <Xm/Xm.h>


typedef void ( *GraphFltCallback ) ( void *, void *, long, long, long, long,
					long, float, Boolean, unsigned, unsigned );


class GraphFltCmd : public NoUndoCmd
{
	public:
		GraphFltCmd ( char *, int active, GraphFltCallback, int, float *, long, long, long, unsigned, void * );
		virtual const char *const className() { return ( "GraphFltCmd" ); }
		void setDataSource ( float *, long, long );
		int redisplay ( long, long, long, long, long, long );

		void setDatumValueBounds ( float, float );
		void setSelection ( long, long );
		void setWindowSize ( int, int );
		int dataChanged() { return ( _dataChanged ); };
		void graphNum ( unsigned int num ) { _graphNumber = num; }
		int graphNum() { return _graphNumber; };
		void centreZero ( Boolean num ) { _centreZero = num; };
		void close() { ok(); }	// external shutdown, preserve info
		void autoClose ( Boolean state ) { _autoClose = state; }
		int startData() { return ( _startData ); }
		
	protected:
		void doCommand();			// called by base class
		GraphFltCallback _callback;	// function to be called
		void *_clientData;			// data provided by caller

		// set pixel RGB values of pixel[n]: setColour(r, g, b, n)
		void setColour ( int, int, int, int );

		// fns called when data in text box is changed
		virtual void field1Changed ( char * );
		virtual void field2Changed ( char * );
		virtual void field3Changed ( char * );
		virtual void field4Changed ( char * );
		virtual void field5Changed ( char * );
		virtual void field7Changed ( char * );

	private:
		/* control data for current view */
		long _field1Value, _field2Value, _field3Value;
		long _field4Value, _field5Value;
		float _field6Value, _field7Value;
				
		// callback for WM 'X'
		static void clientMessageProc ( Widget, XtPointer, XtPointer );

		static void okCallback ( Widget, XtPointer, XtPointer );
		static void cancelCallback ( Widget, XtPointer, XtPointer );
		static void drawTypeCallback ( Widget, XtPointer, XtPointer );
		static void zoomCallback ( Widget, XtPointer, XtPointer );	
 		static void viewAllCallback ( Widget, XtPointer, XtPointer );	
 		static void nextCallback ( Widget, XtPointer, XtPointer );	
 		static void prevCallback ( Widget, XtPointer, XtPointer );	
 		static void expandCallback ( Widget, XtPointer, XtPointer );	
  		static void selRangeCallback ( Widget, XtPointer, XtPointer );	
		static void exposeCallback ( Widget, XtPointer, XEvent *, Boolean * );
		static void resizeCallback ( Widget, XtPointer );

		// scrollbar callbacks
		static void scrolledValueChangedCallback ( Widget, XtPointer, XtPointer );
		static void scrollIncrementCallback ( Widget, XtPointer, XtPointer );
		static void scrollDecrementCallback ( Widget, XtPointer, XtPointer );

		// Callbacks for data change by user
		static void field1ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field2ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field3ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field4ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field5ChangedCallback ( Widget, XtPointer, XtPointer );
		static void field7ChangedCallback ( Widget, XtPointer, XtPointer );

		static void field1EntryCallback ( Widget, XtPointer, XtPointer );
		static void field2EntryCallback ( Widget, XtPointer, XtPointer );
		static void field3EntryCallback ( Widget, XtPointer, XtPointer );
		static void field4EntryCallback ( Widget, XtPointer, XtPointer );
		static void field5EntryCallback ( Widget, XtPointer, XtPointer );
		static void field6EntryCallback ( Widget, XtPointer, XtPointer );
		static void field7EntryCallback ( Widget, XtPointer, XtPointer );
		
		static void startLockChangedCallback ( Widget, XtPointer, XtPointer );
		static void endLockChangedCallback ( Widget, XtPointer, XtPointer );
		static void scaleLockCallback ( Widget, XtPointer, XtPointer );
		static void interpolateCallback ( Widget, XtPointer, XtPointer );
		static void reloadCallback ( Widget, XtPointer, XtPointer );

		// 'confirm new datum value' messageDialogManager callbacks
		static void datumChangeOkCallback ( XtPointer );
		static void datumChangeCancelCallback ( XtPointer );

	        // pointer functions
        	static void pointerMotionCallback ( Widget, XtPointer, XEvent *, Boolean * );
        	static void button3MotionCallback ( Widget, XtPointer, XEvent *, Boolean * );
        	static void mouseButtonPressCallback ( Widget, XtPointer, XEvent *, Boolean * );
        	static void mouseButtonReleaseCallback ( Widget, XtPointer, XEvent *, Boolean * );

		Boolean ok();
		void cancel();
		void zoom ( void );
		void selRange ( void );
		void redraw();
		void redrawSelection ( long, long );
		int checkTextEntry ( char * );
		void reload();
		void pointerMotion ( void );
		void button3Motion ( void );
		void mouseButton1Press ( void );
		void mouseButton3Press ( void );
		void mouseButtonRelease ( int );
		void clientMessage ( XmAnyCallbackStruct * );
		void scrolledValueChanged ( int );

		void changeField1 ( long );
		void changeField2 ( long );
		void changeField3 ( long );
		void changeField4 ( long );
		void changeField5 ( long );
		void changeField6 ( float );
		void changeField7 ( float );
		
		void field1Entry ( long );
		void field2Entry ( long );
		void field3Entry ( long );
		void field4Entry ( long );
		void field5Entry ( long );
		void field7Entry ( float );
		
		void acquireColour ( int );
		void releaseColour ( int );
		void dragYScale ( int );
		void datumChangeOk ( long );
		void checkSelection ( void );
		void getGeometry ( void );
		void refreshView ( void );
		void viewAll ( void );
		void next ( void );
		void prev ( void );
		void expand ( void );

		void setZoomLabel ( Boolean );
		long datumNum ( long );
		float datumValue ( long );
		void setScrollBar ( int, int, int, Boolean );

		Cursor _normalCursor;
		Cursor _busyCursor;
		void setNormalCursor ( void );
		void setBusyCursor ( void );

		Boolean _drawPoints;		// False = draw lines
		Boolean _refreshView;
		Boolean _reload;			// indicator in callback to calling code
		Boolean _zoomStatus;		// flag to indicate zoom status
		Boolean _reloading;
		Boolean _closeDown;
		Boolean _startSelectionLock;
		Boolean _endSelectionLock;
		Boolean _drawing;
		Boolean _centreZero;
		Boolean _lockScale;
		Boolean _interpolate;
		Boolean _autoClose;

		char _applicationName[128];
		char _name[128];
		long _maxData;				// max data in data source
		float _minY, _maxY;			// datum value limits
		long _numData;				// number of data points loaded into array
		long _oldStart, _oldEnd;	// snap back from zoom
		long _lastDrawStart;		// prev. screenX for interpolation on draw
		long _startData;			// start of data in array (offset into file)
		int _xCoord, _yCoord;
		float *_data;				// data to be displayed
		int _colourAcquired;
		int _dataChanged;
		unsigned int _selecting;	// indicate redraw if pointer dragged
		unsigned int _numDigits;
		unsigned int _windowHeight;
		unsigned int _windowWidth;
		unsigned int _graphNumber;	// id of this graph
		unsigned int _callbackType;	// 0 = report/reload, 1 = 'ok', 2 = 'cancel' '
		unsigned _r, _g, _b;
		Pixel _pixel[3];       		// [1] is drawing colour, [0] is background

		Widget _canvas;
		Widget _frame;
		Widget _popup, _rc0, _rc1, _rca, _rcb, _sep;
		Widget _field1; /* output area */
		Widget _field2;
		Widget _field3;
		Widget _field4;
		Widget _field5; /* current (default or pointer) x value */
		Widget _field6; /* current (default or pointer) y value */
		Widget _field7; /* y scaling factor */

		Widget _label1; /* label for output area */
		Widget _label2;
		Widget _label3;
		Widget _label4;
		Widget _label5;
		Widget _label6;
		Widget _label7;

		/* command widgets */
		Widget _label, _ok, _cancel, _drawType, _zoom, _viewAll, _next, _prev;
		Widget _startSelLock, _endSelLock, _scrollBar, _selRange, _expand;
		Widget _scaleLock, _interp;
};

#endif


/*
  Notes:

  Warning!
  Inexplicable crashes are usually due to a current x datum pointer which is
  outside the range of the start and end of the data. This can happen if the
  code updating the view caches the datum pointer along with the range and
  selection pointers which it receives in the callback function specified for
  the graphCmd instance. If the calling code causes the data to be changed and
  then calls the view redisplay method with the old x current pointer, if this
  happens to be out of range of the new data start and end, access to an
  element off one end or other of the data array occurs. If in doubt, when
  calling the redisplay method, pass the current x datum value as the start
  selection value or start range value instead (setting the current x datum
  number to either the start of the view or the start of the selection in
  the view)

  The graph as is can only cope with displaying floating point numbers in an
  array. These must be supplied by the calling routine.  

  Usage:

	GraphFltCmd *graphCmd = new GraphFltCmd ( pWave->name(), TRUE,
									&graphCmdCallback,
									pointerToIntDataArray,
									offsetIntoDataArray,
									numberOfIntDataItems,
									maximumExternalDataItems,
									graphID,
									( void * ) this );

	graphCmd->setDatumValueBounds ( -32768, 32767 );	// arbitrary bounds
	graphCmd->execute();

	
	Public methods:
	
		setDataSource ( dataPointer, numIntDataItems );
		Used to supply new data to the graph viewer instance after its initial
		invocation. Data should be an integer array.
		
		setDatumValueBounds ( min, max );
		Set magnitude limits of data; used to check datum value change by the
		user.
		
		int dataChanged()
		Per loaded data block, indicates status of data modification. This is
		reset for each loaded block and only set once per changed datum in a
		block. Allows calling code to rewrite data block if change acceptable.

		redisplay();
		Provision for calling code to invoke a display refresh, e.g. if the
		selected data has been modified.		

		int graphNum()
		Returns an ID number for the associated display, allocated via the
		graphCmd constructor. This ID is also returned via GraphCallback, so
		the calling code can determine to which data set the returned range and
		selection pointers refer.

	Controls:
	
		Text boxes are interactive, both accepting and indicating data.
		addtionally, mouse pointer control is available
		
		Left click and drag (left or right) reloads data if there is more
		available externally than is currently displayed Note that calling code
		should check whether data has been changed and if necessary rewritten
		before a reload (via the dataChanged() method).
		
		Vertical left click and drag changes the scaling factor applied to the
		display. Click the mouse button within the central 50% of the view and
		drag up or down past this point. Scaling may be reset to unity by
		dragging back to the central portion of the display. Alternatively, a
		scaling factor may be entered manually by clicking in the y scale
		display box and entering a new value; this activated on pressing
		return.
		

	Feedback.
		Information about the range and selection values is continuously passed
		back to the calling code, and also when the 'ok' button is activated.
		The calling code should declare a callback function (here denoted
		graphCmdCallback)
		
		static void graphCmdCallback
			( void *cmd, void *clientData, long start, long end,
				long selStart, long selEnd, Boolean reload, int graphNumber,
				int callbackReason )
			
		and also a method to which the callback passes control data:
		
		void graphCmdCallbackFinished
 			( GraphFltCmd *gc, long start, long end,
				long selStart, long selEnd, Boolean reload, int graphNumber,
				int callbackReason );
		
		which is implemented so as to calculate the necessary parameters for a
		reload if required. Since it will not be known in advance whether the
		callback has been activated due to a request for data from the range or
		selection controls, the calling code must determine the size of
		the data block to be loaded, as in the following example:
		
		min = start;

		if ( end < min )
			min = end;
		
		if ( selStart != selEnd )
		{
			if ( selStart < min )
				min = selStart;
	
			if ( selEnd < min )
				min = selEnd;
		}

		max = start;

		if ( end > max )
			max = end;

		if ( selStart != selEnd )
		{
			if ( selStart > max )
				max = selStart;
	
			if ( selEnd > max )
				max = selEnd;
		}

		An integer array of size ( max - min + 1 ) * 2 bytes must be created.
		Data in this array is then passed back to the viewer object:
		
		if (reload == True)
			setDataSource ( pointerToDataArray, max - min + 1 );
			
		If the reload flag is false, the current range and selection values may
		be used to perform operations on the current data.
		
		The callbackReason parameter is used since the callback is continually
		invoked to provide a stream of information to the calling code, and is
		also invoked on 'cancel' or 'quit'. The returned value indicates the
		reason for the call:
		
		0 default, reload may be true or false indicating request for new data
		or just a report back on status;
		
		1 'Ok' activated - in this case reload is false, and the returned
		values are those currently displayed in the graph on closedown.
		
		2 'Cancel' activated - again, reload is false, and the returned values
		for range and selection can be disregarded. 
		
		See lessTifApp/demo/Mixer/EditWaveCmd for a practical implememtation.
*/
