/*
 WorkingDialogManager.h - add features to DialogManager
 modified by Will 2000 - added setTimeout, updateTimeout, setTitle, _text

 Adapted from
 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall
*/

#ifndef WORKINGDIALOGMANAGER_H
#define WORKINGDIALOGMANAGER_H

#include "DialogManager.h"


class PixmapCycler;

class WorkingDialogManager : public DialogManager
{
	public:
		WorkingDialogManager ( const char * );
		virtual Widget post ( const char *, void *clientData = NULL,
							  DialogCallback ok = NULL,
							  DialogCallback cancel = NULL,
							  DialogCallback help = NULL );
		void unpost();
		void updateMessage ( char * );
		void updateTimeout ( unsigned long );

	protected:
		Widget createDialog ( Widget );
		PixmapCycler *_busyPixmaps;	// source of animation sequence
		XtIntervalId _intervalId;	// ID of last timeout
		static void unpostCallback ( Widget, XtPointer, XtPointer );
		static void timerCallback ( XtPointer, XtIntervalId * );
		void timer();
		
	private:
		unsigned long _updateInterval;	// update progress bar interval, mS
		unsigned long _timerInterval;
		void setTimeout ( void );		// update timer callback timeout
		char _text[7];
};

extern WorkingDialogManager *workingDialogManager;

#endif

/*
  Notes:
  
  The LessTif Application Framework supports a single globally-available instance
  of WorkingDialogManager, 'workingDialogManager'. Applications using this class
  include the class header file and send messages to workingDialogManager.
  
  Supports an updateMessage() function allopwing text to be changed whilst dialog
  is posted. Applications can remove the dialog with unpost(). The status of a
  busy app is reported.
  
  Supports an updateTimeout(timeout) function allowing the interval during which
  the dialog progress bar is updated to be defined by the user program. The value
  supplied should be calculated as a percentage of the total process time taken
  by the process being monitored by the dialog.
  
  User input is locked out while the app is busy and the dialog is posted.

  The private _text variable is used by setTimemout() to dynamically calculate a
  new _updateInterval so the progress bar can be synchronised to the file
  processing progress.

*/
